#!/bin/sh

. ../base.lib.sh
. ../environment.lib.sh
. ../bool.lib.sh
. ../number.lib.sh
. ../fs.lib.sh
. ../string.lib.sh

write_exp_test_worker() {
	local var file mode

	mode="$1"
	shift
	file="$(mktemp)"

	printf -- '#!/bin/sh -eu\n\n' > "${file}"
	chmod +x "${file}"

	for var in "$@"; do
		if [ "${mode}" = "export" ]; then
			printf -- 'test_export_%s() {\n\tprintf -- "%%s\\n" "${%s}"\n}\n\n' "$var" "$var" >> "${file}"
		elif [ "${mode}" = "unexport" ]; then
			printf -- 'test_unexport_%s() {\n\tprintf -- "%%s\\n" "${%s+unset}"\n}\n\n' "$var" "$var" >> "${file}"
		fi
	done

	printf -- '\ncase "$1" in\n' >> "${file}"

	for var in "$@"; do
		if [ "${mode}" = "export" ]; then
			printf -- '\t(%s)\n\t\ttest_export_%s\n\t;;\n\n' "${var}" "${var}" >> "${file}"
		elif [ "${mode}" = "unexport" ]; then
			printf -- '\t(%s)\n\t\ttest_unexport_%s\n\t;;\n\n' "${var}" "${var}" >> "${file}"
		fi
	done

	printf -- 'esac\n\n' >> "${file}"

	printf -- '%s' "${file}"
}

write_export_tester() {
	write_exp_test_worker export "$@"
}

write_unexport_tester() {
	write_exp_test_worker unexport "$@"
}

test_k_env_export (){
	local file
	file="$(write_export_tester TEST_VAR)"
	TEST_VAR='foo'
	k_env_export TEST_VAR
	assertEquals 'TEST_VAR is foo' "$TEST_VAR" "$(${file} TEST_VAR)"
}

test_k_env_import_apache (){
	k_env_import_apache ./test-apache.conf
	assertEquals "FOO is BAR" "BAR" "${FOO}"
	assertEquals "BAZ is BOO" "BOO" "${BAZ}"
}

test_k_env_export_apache (){
	file="$(write_export_tester FOO BAZ)"
	k_env_import_apache ./test-apache.conf
	k_env_export_apache

	assertEquals "FOO is BAR" "BAR" "$(${file} FOO)"
	assertEquals "BAZ is BOO" "BOO" "$(${file} BAZ)"
}

test_k_env_unset_apache (){
	k_env_import_apache ./test-apache.conf
	k_env_unset_apache

	assertEquals "FOO is unset" "" "${FOO+unset}"
	assertEquals "BAZ is unset" "" "${BAZ+unset}"
}

test_k_env_unexport_apache (){
	local file

	file="$(write_unexport_tester FOO BAZ)"
	k_env_import_apache ./test-apache.conf
	k_env_export_apache
	k_env_unexport_apache

	assertEquals "FOO is BAR" "" "$(${file} FOO)"
	assertEquals "BAZ is BOO" "" "$(${file} BAZ)"
}

gecos_set_known() {
	sudo chfn --other shell-script-lib-test@koalephant.com --full-name "KSSL User" --home-phone 12345 --work-phone 67890 --room 9876 "${USER}"
}

test_k_env_import_gecos (){
	gecos_set_known
	k_env_import_gecos
	assertEquals "NAME is KSSL User" "KSSL User" "${NAME}"
	assertEquals "ROOM is 9876" "9876" "${ROOM}"
	assertEquals "TELEPHONE_WORK is 67890" "67890" "${TELEPHONE_WORK}"
	assertEquals "TELEPHONE_HOME is 12345" "12345" "${TELEPHONE_HOME}"
	assertEquals "EMAIL is shell-script-lib-test@koalephant.com" "shell-script-lib-test@koalephant.com" "${EMAIL}"
}

test_k_env_export_gecos (){
	local file

	file="$(write_export_tester NAME ROOM TELEPHONE_WORK TELEPHONE_HOME EMAIL)"
	gecos_set_known
	k_env_import_gecos
	k_env_export_gecos
	assertEquals "NAME is KSSL User" "KSSL User" "$(${file} NAME)"
	assertEquals "ROOM is 9876" "9876" "$(${file} ROOM)"
	assertEquals "TELEPHONE_WORK is 67890" "67890" "$(${file} TELEPHONE_WORK)"
	assertEquals "TELEPHONE_HOME is 12345" "12345" "$(${file} TELEPHONE_HOME)"
	assertEquals "EMAIL is shell-script-lib-test@koalephant.com" "shell-script-lib-test@koalephant.com" "$(${file} EMAIL)"
}

test_k_env_unset_gecos (){
	gecos_set_known
	k_env_import_gecos
	k_env_unset_gecos
	assertEquals "NAME is unset" "" "${NAME+unset}"
	assertEquals "ROOM is unset" "" "${ROOM+unset}"
	assertEquals "TELEPHONE_WORK is unset" "" "${TELEPHONE_WORK+unset}"
	assertEquals "TELEPHONE_HOME is unset" "" "${TELEPHONE_HOME+unset}"
	assertEquals "EMAIL is unset" "" "${EMAIL+unset}"
}

test_k_env_unexport_gecos (){
	local file

	file="$(write_unexport_tester NAME ROOM TELEPHONE_WORK TELEPHONE_HOME EMAIL)"
	gecos_set_known
	k_env_import_gecos
	k_env_export_gecos
	k_env_unset_gecos

	assertEquals "NAME is unset" "" "$(${file} NAME)"
	assertEquals "ROOM is unset" "" "$(${file} ROOM)"
	assertEquals "TELEPHONE_WORK is unset" "" "$(${file} TELEPHONE_WORK)"
	assertEquals "TELEPHONE_HOME is unset" "" "$(${file} TELEPHONE_HOME)"
	assertEquals "EMAIL is unset" "" "$(${file} EMAIL)"
}

test_k_env_safename (){
	assertEquals "cannot have spaces" "cannothavespaces" "$(k_env_safename "cannot have spaces")"
}



test_k_env_export_ldap (){
	printf -- 'Empty Test\n' >&2
}



test_k_env_import_ldap (){
	printf -- 'Empty Test\n' >&2
}



test_k_env_set (){
	printf -- 'Empty Test\n' >&2
}



test_k_env_unexport (){
	printf -- 'Empty Test\n' >&2
}



test_k_env_unset (){
	printf -- 'Empty Test\n' >&2
}



test_k_env_unset_ldap (){
	printf -- 'Empty Test\n' >&2
}

